#include <iostream>
#include <sstream>
#include <vector>
#include "DynamicUpdateControl.h"

using namespace std;

//prototypes and consts for some utilities
const string whitespace = " \t\n";
void trim(string& str);
void split(const string& str, vector<string>& tokens, const string& delimiters =
		whitespace);
void splitFirstNTokens(const string& str, vector<string>& tokens, unsigned int n, const string& delimiters = whitespace);

//Dynamic Update Control public function definitions
DynamicUpdateControl::DynamicUpdateControl(ostream& outputStream) :
	out(outputStream) {
}

DynamicUpdateControl::~DynamicUpdateControl() {
	map<string, BaseHandle*>::iterator iter = ptrTbl.begin();
	while (iter != ptrTbl.end()) {
		delete iter->second;
		iter++;
	}
}

void DynamicUpdateControl::processMessage(const string& msg) {
	// this could prolly be done more prettily, but whatever. this aint
	// the meat of the class.
	vector<string> tokens;
	splitFirstNTokens(msg, tokens, 1);

	string& t1 = tokens[0];
	string& ts = tokens[1];

	if(*t1.begin() == '#'){
		vector<string> commandArgs;
		splitFirstNTokens(ts, commandArgs, 1);
		string& cmd = t1;
		string& arg1 = commandArgs[0];
		string& arg2 = commandArgs[1];
		if (0 == cmd.compare("#open")) {
			if(arg1.empty()){
				throw MalformedMessage();
			}else{
				//ignore arg2
				openParameterFile(arg1);
			}
		} else if (0 == cmd.compare("#save")) {
			if(arg1.empty()){
				throw MalformedMessage();
			}else{
				//ignore arg2
				saveParameterFile(arg1);
			}
		} else if (0 == cmd.compare("#set")) {
			if(arg1.empty()){
				throw MalformedMessage();
			}else if(arg2.empty()){
				updateParameter(cwp,arg1);
			}else{
				updateParameter(arg1,arg2);
			}
		} else if (0 == cmd.compare("#show")) {
			if(arg1.empty()){
				showParameter(cwp);
			}else{
				showParameter(arg1);
			}
		} else if (0 == cmd.compare("#showAll")) {
			// ignore args, show all
			showAllParameters();
		} else {
			throw UnrecognizedCommand();
		}
	}else if(!t1.empty()){
		if(ts.empty()){
			updateParameter(cwp, t1);
		}else{
			updateParameter(t1, ts);
		}
		showParameter(cwp);
	}else{
		throw MalformedMessage();
	}
/*
	vector<string> tokens;
	split(msg, tokens);

	if (tokens.size() == 0)
		throw MalformedMessage();

	if (*tokens[0].begin() == '#') {
		string& cmd = tokens[0];
		if (0 == cmd.compare("#open")) {
			openParameterFile(tokens[0]);
		} else if (0 == cmd.compare("#save")) {
			saveParameterFile(tokens[0]);
		} else if (0 == cmd.compare("#set")) {
			if (tokens.size() == 1) {
				updateParameter(cwp, tokens[0]);
			} else if (tokens.size() == 2) {
				updateParameter(tokens[0], tokens[1]);
			} else {
				throw MalformedMessage();
			}
		} else if (0 == cmd.compare("#show")) {
			if (tokens.size() == 1) {
				showParameter(cwp);
			} else if (tokens.size() == 2) {
				showParameter(tokens[1]);
			} else {
				throw MalformedMessage();
			}
		} else if (0 == cmd.compare("#showAll")) {
			if (tokens.size() == 1) {
				showAllParameters();
			} else {
				throw MalformedMessage();
			}
		} else {
			throw UnrecognizedCommand();
		}
	} else {
		if (tokens.size() == 1) {
			updateParameter(cwp, tokens[0]);
		} else if (tokens.size() == 2) {
			updateParameter(tokens[0], tokens[1]);
		} else {
			throw MalformedMessage();
		}
		showParameter(cwp);
	}
	*/
}

void DynamicUpdateControl::updateParameter(const string& prmName,
		const string& prmValue) {
	if (ptrTbl.count(prmName) == 0)
		throw UnregisteredParameter();
	ptrTbl[prmName]->read(prmValue);
	cwp = prmName;
}

void DynamicUpdateControl::openParameterFile(const string& filename) {
	//needs implementation
}

void DynamicUpdateControl::saveParameterFile(const string& filename) {
	//needs implementation
}

void DynamicUpdateControl::showParameter(const string& prmName) {
	if (ptrTbl.count(prmName) == 0)
		throw UnregisteredParameter();
	out << prmName << " " << ptrTbl[prmName]->toString() << endl;
	cwp = prmName;
}

void DynamicUpdateControl::showAllParameters() {
	map<string, BaseHandle*>::iterator iter = ptrTbl.begin();
	while (iter != ptrTbl.end()) {
		string prmName = iter->first;
		out << prmName << " " << ptrTbl[prmName]->toString() << endl;
		iter++;
	}
}

// utility functions used above
void trim(string& str) {
	{
		size_t startpos = str.find_first_not_of(whitespace);
		size_t endpos = str.find_last_not_of(whitespace);

		if (string::npos == startpos || string::npos == endpos) {
			str = "";
		} else {
			str = str.substr(startpos, endpos - startpos + 1);
		}
	}
	{
		size_t startpos = str.find_first_not_of(whitespace);
		if (string::npos != startpos) {
			str = str.substr(startpos);
		}
	}
	{
		size_t endpos = str.find_last_not_of(whitespace);
		if (string::npos != endpos) {
			str = str.substr(0, endpos + 1);
		}
	}
}

void split(const string& str, vector<string>& tokens, const string& delimiters) {
	string::size_type lastPos = str.find_first_not_of(delimiters, 0);
	string::size_type pos = str.find_first_of(delimiters, lastPos);

	while (string::npos != pos || string::npos != lastPos) {
		tokens.push_back(str.substr(lastPos, pos - lastPos));
		lastPos = str.find_first_not_of(delimiters, pos);
		pos = str.find_first_of(delimiters, lastPos);
	}
}

void splitFirstNTokens(const string& str, vector<string>& tokens, unsigned int n, const string& delimiters){
	string::size_type lastPos = str.find_first_not_of(delimiters, 0);
	string::size_type pos = str.find_first_of(delimiters, lastPos);

	for(unsigned int i = 0; i < n; i++){
		if(string::npos != pos || string::npos != lastPos){
			tokens.push_back(str.substr(lastPos, pos - lastPos));
			lastPos = str.find_first_not_of(delimiters, pos);
			pos = str.find_first_of(delimiters, lastPos);
		}else{
			tokens.push_back("");
		}
	}
	if(lastPos != string::npos){
		tokens.push_back(str.substr(lastPos));
	}

}

int main(int argc, const char* argv[]){
	string sPrm = "";
	bool bPrm = false;
	float fPrm = 0.0f;

	DynamicUpdateControl duc;
	duc.registerParameter<string>("myString",&sPrm);
	duc.registerParameter<float>("myFloat",&fPrm);
	duc.registerParameter<bool>("myBool",&bPrm);

	duc.processMessage("myBool true");
	duc.processMessage("myString show me the money");
	duc.processMessage("myFloat 3.14");

	cout << "Done." << endl;
	cin.get();
	return 0;
}
